@@ -8,7 +8,7 @@ use wgpu::CommandBuffer;
 
 use crate::pipeline::{Pipeline, Vertex};
 
-/// Wrapper over [`glyph_brush::GlyphBrush`].
+/// Wrapper over [`glyph_brush::GlyphBrush`]. Draws text.
 ///
 /// Used for queuing and rendering text with [`TextBrush::queue`] and [`TextBrush::draw_queued`].
 pub struct TextBrush<F = FontArc, H = DefaultSectionHasher> {
@@ -17,6 +17,10 @@ pub struct TextBrush<F = FontArc, H = DefaultSectionHasher> {
 }
 
 impl<F: Font + Sync, H: BuildHasher> TextBrush<F, H> {
+    /// Queues section for drawing. This should be called every frame for every section that is going to be drawn.
+    ///
+    /// This can be called multiple times for different sections that want to use the
+    /// same font and gpu cache.
     #[inline]
     pub fn queue<'a, S>(&mut self, section: S)
     where
@@ -25,6 +29,7 @@ impl<F: Font + Sync, H: BuildHasher> TextBrush<F, H> {
         self.inner.queue(section);
     }
 
+    /// Draws all queued text and sections with [`queue`](#method.queue) function.
     pub fn draw_queued(
         &mut self,
         device: &wgpu::Device,
@@ -41,8 +46,26 @@ impl<F: Font + Sync, H: BuildHasher> TextBrush<F, H> {
 
             match brush_action {
                 Ok(_) => break,
+
+                // If texture is too small use BrushBuilder::initial_cache_size
+                // because resizing texture should be avoided.
                 Err(BrushError::TextureTooSmall { suggested }) => {
-                    self.pipeline.resize_texture(suggested.0, suggested.1)
+                    if log::log_enabled!(log::Level::Warn) {
+                        log::warn!("Resizing cache texture! This should be avoided by building \
+                        TextBrush with BrushBuilder::initial_cache_size() and providing cache texture dimensions.");
+                    }
+                    let max_image_dimension = device.limits().max_texture_dimension_2d;
+                    let (width, height) = if (suggested.0 > max_image_dimension
+                        || suggested.1 > max_image_dimension)
+                        && (self.inner.texture_dimensions().0 < max_image_dimension
+                            || self.inner.texture_dimensions().1 < max_image_dimension)
+                    {
+                        (max_image_dimension, max_image_dimension)
+                    } else {
+                        suggested
+                    };
+                    self.pipeline.resize_texture(device, width, height);
+                    self.inner.resize_texture(width, height);
                 }
             }
         }
@@ -55,6 +78,10 @@ impl<F: Font + Sync, H: BuildHasher> TextBrush<F, H> {
         self.pipeline.draw(device, view)
     }
 
+    /// Resizes text rendering pipeline.
+    ///
+    /// Run this function whenever the surface is resized.
+    /// _width_ and _height_ should be **surfaces** dimensions.
     pub fn resize(&mut self, width: f32, height: f32, queue: &wgpu::Queue) {
         self.pipeline.resize(width, height, queue);
     }
@@ -66,17 +93,27 @@ pub struct BrushBuilder<F, H = DefaultSectionHasher> {
 }
 
 impl BrushBuilder<()> {
+    /// Creates a [`BrushBuilder`] with [`Font`].
     #[inline]
     pub fn using_font<F: Font>(font: F) -> BrushBuilder<F> {
         BrushBuilder::using_fonts(vec![font])
     }
 
+    /// Creates a [`BrushBuilder`] with font byte data.
     #[inline]
     pub fn using_font_bytes(data: &[u8]) -> Result<BrushBuilder<FontRef>, InvalidFont> {
         let font = FontRef::try_from_slice(data)?;
         Ok(BrushBuilder::using_fonts(vec![font]))
     }
 
+    /// Creates a [`BrushBuilder`] with multiple fonts byte data.
+    #[inline]
+    pub fn using_font_bytes_vec(data: &[u8]) -> Result<BrushBuilder<FontRef>, InvalidFont> {
+        let font = FontRef::try_from_slice(data)?;
+        Ok(BrushBuilder::using_fonts(vec![font]))
+    }
+
+    /// Creates a [`BrushBuilder`] with multiple [`Font`].
     pub fn using_fonts<F: Font>(fonts: Vec<F>) -> BrushBuilder<F> {
         BrushBuilder {
             inner: glyph_brush::GlyphBrushBuilder::using_fonts(fonts),
@@ -87,6 +124,7 @@ impl BrushBuilder<()> {
 impl<F: Font, H: BuildHasher> BrushBuilder<F, H> {
     glyph_brush::delegate_glyph_brush_builder_fns!(inner);
 
+    /// Builds a [`TextBrush`] consuming [`BrushBuilder`].
     pub fn build(
         self,
         device: &wgpu::Device,
